@@ -6,12 +6,12 @@
 
 # Improvements: Set the corresponding threshold for each selected bandwidth, i.e. fix the bandwidth if the change of the selected bandwidths is less than threshold
 gw.fitted <- function(X, beta) {
-  .Call(`GWmodel_fitted`, X, beta)
+  fitted(X, beta)
 }
 
-gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, criterion="dCVR", max.iterations=2000,threshold=0.00001, dMats, p.vals, theta.vals,longlat=FALSE,
-                           bws0, bw.seled=rep(F, length(bws0)), approach = "AIC", bws.thresholds=rep(0.1, length(dMats)), bws.reOpts=5, verbose=F, hatmatrix=T, 
-                           predictor.centered=rep(T, length(bws0)-1),nlower = 10)
+gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, criterion="dCVR", max.iterations=2000,threshold=0.00001, dMats, var.dMat.indx, p.vals, theta.vals,longlat=FALSE,
+                           bws0, bw.seled, approach = "AIC", bws.thresholds, bws.reOpts=5, verbose=F, hatmatrix=T, 
+                           predictor.centered=rep(T, length(bws0)-1),nlower = 10, parallel.method=F,parallel.arg=NULL)
 {
   ##Record the start time
   timings <- list()
@@ -54,34 +54,45 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
   lms$y <- y
   #colnames(x)[1]<-"Intercept"
   #################################################
+  if(missing(bw.seled))
+  {
+    bw.seled <- rep(F, var.n)
+  }
+  else {
+     bw.seled <- rep_len(bw.seled, var.n)
+  }
+  if(missing(bws.thresholds))
+  {
+    bws.thresholds <- rep(0.1, var.n)
+  }
+  else {
+     bws.thresholds<- rep_len(bws.thresholds, var.n)
+  }
   ##########Centering and scaling predictors
-  if(!is.na(idx1))
+  if(missing(predictor.centered))
   {
-    if(length(predictor.centered)!=var.n-1)
-    {
-      predictor.centered <- rep(T, length.out=var.n-1)
-      warning("All the predictors will be centered, please check the parameter predictor.centered")
-    }
-    # if(length(predictor.scaled)!=var.n-1)
-    # {
-    #   predictor.scaled <- rep(F, length.out=var.n-1)
-    #   warning("All the predictors will not be scaled, please check the parameter predictor.scaled")
-    # }
+    predictor.centered <- rep(T, length.out=var.n-1)
   }
-  else
-  {
-    if(length(predictor.centered)!=var.n)
-    {
-      predictor.centered <- rep(T, length.out=var.n)
-      warning("All the predictors will be centered, please check the parameter predictor.centered")
-    }
-    # if(length(predictor.scaled)!=var.n-1)
-    # {
-    #   predictor.scaled <- rep(F, length.out=var.n-1)
-    #   warning("All the predictors will not be scaled, please check the parameter predictor.scaled")
-    # }
+  else {
+     if(!is.na(idx1))
+     {
+       if(length(predictor.centered)!=var.n-1)
+       {
+         predictor.centered <- rep(T, length.out=var.n-1)
+         warning("All the predictors will be centered, please check the parameter predictor.centered")
+       }
+     }
+     else
+     {
+       if(length(predictor.centered)!=var.n)
+       {
+         predictor.centered <- rep(T, length.out=var.n)
+         warning("All the predictors will be centered, please check the parameter predictor.centered")
+       }
+     }
   }
   
+  
   n.cent <- length(which(predictor.centered))
   if(!is.na(idx1))
   {
@@ -125,31 +136,33 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     dMats <- list()
     if(missing(p.vals))
     {
-      p.vals <- 2
       dMat <- gw.dist(dp.locat,longlat=longlat)
-      for(i in 1:var.n)
-        dMats[[i]] <- dMat
+      dMats[[1]] <- dMat
+	  var.dMat.indx <- rep(1, var.n)
     }
     else
     {
       p.vals <- rep_len(p.vals, var.n)
       if(missing(theta.vals))
-        theta.vals <- rep_len(0, var.n)
+	    {
+	      theta.vals <- rep_len(0, var.n)
+	    } 
       else
         theta.vals<- rep_len(theta.vals, var.n)
       for(i in 1:var.n)
       {
         dMats[[i]] <- gw.dist(dp.locat, p=p.vals[i], theta=theta.vals[i],longlat=longlat)
       }
+	    var.dMat.indx <- 1:var.n
     }
   }
   else if(is.list(dMats))
   {
-    if(length(dMats)!=var.n)
-      stop("Please specify a distance matrix for each independent variable!")
+    if(length(dMats) < max(var.dMat.indx))
+      stop("Please specify a distance matrix index via the var.dMat.indx for each independent variable!")
     else
     {
-      for(i in 1:var.n)
+      for(i in 1:length(dMats))
       {
         dim.dMat<-dim(dMats[[i]])
         if (dim.dMat[1]!=dp.n||dim.dMat[2]!=dp.n)
@@ -177,7 +190,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
       }
       cat("Now select an optimum bandwidth for the model: ", fml,"\n")
       part1<-paste("bws0[i]<-bw.gwr(",fml,sep="")
-      part2<-"data=regression.points,kernel=kernel,approach=approach,dMat=dMats[[i]])"
+      part2<-"data=regression.points,kernel=kernel,approach=approach,adaptive=adaptive,dMat=dMats[[var.dMat.indx[i]]], parallel.method=parallel.method,parallel.arg=parallel.method)"
       expression<-paste(part1,part2,sep=",")
       print(expression)
       eval(parse(text=expression))
@@ -195,7 +208,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
   cat("------   Calculate the initial beta0 from the above bandwidths    ------\n")
   #dMat <- gw.dist(dp.locat=dp.locat, p=2, theta=0, longlat=longlat)
   dMat <- dMats[[1]]
-  bw.int0 <- bw.gwr1(x1, y, dp.locat,approach=approach,kernel=kernel,adaptive=adaptive,dMat,verbose=verbose, nlower = nlower)
+  bw.int0 <- bw.gwr2(x1, y, dp.locat,approach=approach,kernel=kernel,adaptive=adaptive,dMat,verbose=verbose, nlower = nlower, parallel.method=parallel.method,parallel.arg=parallel.method)
   #betas <- gwr.q(x, y, dp.locat, adaptive=adaptive, bw=bw.int0, kernel=kernel,dMat=dMat)
   #######Re-initialize the betas by a simple back-fitting process
   #betas <- gwr.backfit(x, y, betas,dp.locat,dp.locat, FALSE,criterion, adaptive, bws0, kernel,dMats, max.iterations,threshold*100)
@@ -230,7 +243,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
   criterion.val <- 10000000  
   #yhat.i <- betas*x
   #print(yhat.i)
-  resid.i <- y - gw.fitted(x1, betas)
+  resid.i <- y - fitted(x1, betas)
   RSS0 <- sum(resid.i^2)
   RSS1 <- 0
   RSS.vals <- c(RSS0, RSS1, criterion.val)
@@ -241,7 +254,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     cat("    Iteration ", ieration+1, ":\n")   
     for(i in 1:var.n)
     {
-      dMat <- dMats[[i]]  
+      dMat <- dMats[[var.dMat.indx[i]]]  
       f.i <- betas[,i]*x1[,i]
       y.i <- resid.i + f.i
       if(bw.seled[i])
@@ -249,7 +262,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
       else
       {
         cat("Now select an optimum bandwidth for the variable: ", InDevars[i], "\n")
-        bw.i <- bw.gwr1(matrix(x1[, i], ncol = 1), y.i, dp.locat, approach = approach, kernel = kernel, adaptive = adaptive, dMat, verbose = verbose, nlower = nlower)
+        bw.i <- bw.gwr2(matrix(x1[, i], ncol = 1), y.i, dp.locat, approach = approach, kernel = kernel, adaptive = adaptive, dMat, verbose = verbose, nlower = nlower,parallel.method=parallel.method,parallel.arg=parallel.arg)
         cat("The newly selected bandwidth for variable ", InDevars[i])
         cat(" is: ", bw.i, "\n")
         cat("The bandwidth used in the last ieration is: ", bws0[i])
@@ -291,13 +304,13 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
       #AICcs[i] <- gwr.aic(bw.i, matrix(x[,i], ncol=1), y.i, kernel, adaptive, dp.locat, dMat=dMat, verbose=F)
       #yhat.i[,i] <- betai*x[,i]
       betas[,i] <- betai
-      resid.i <- y - gw.fitted(x1, betas)
+      resid.i <- y - fitted(x1, betas)
       #resid.i <- ehat(y.i, matrix(x[,i], ncol=1), matrix(betas[,i], ncol=1))
       #betas[,i] <- betai
     }
     bws.vars <- rbind(bws.vars, bws0)
     #AICc.vals <- rbind(AICc.vals, AICcs) 
-    RSS1 <- sum((y - gw.fitted(x1, betas))^2)   
+    RSS1 <- sum((y - fitted(x1, betas))^2)   
     if(criterion=="CVR")
     {
       criterion.val <- abs(RSS1-RSS0)
@@ -314,26 +327,40 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     ieration <- ieration+1            
   }
   #####Output
-  yhat <- gw.fitted(x1, betas)
+  yhat <- fitted(x1, betas)
   residual <- y - yhat
-  ################
-  RSS.gw <- RSS1
-  sigma.hat21 <- RSS.gw/dp.n
-  yss.g <- sum((y - mean(y))^2)
-  R2.val <-  1-RSS.gw/yss.g
-  R2adj <- NA
-  AICc <- NA
+  GW.diagnostic <- NA
+  ###############
+  # RSS.gw <- RSS1
+  # sigma.hat21 <- RSS.gw/dp.n
+  # yss.g <- sum((y - mean(y))^2)
+  # R2.val <-  1-RSS.gw/yss.g
+  # R2adj <- NA
+  # AICc <- NA
+  # if(hatmatrix)
+  # {
+    # tr.Shat <- sum(diag(Shat))
+    # tr.StShat<-sum(Shat^2)
+    # edf<- dp.n - 2*tr.Shat + tr.StShat
+    # AICc <- dp.n*log(sigma.hat21) + dp.n*log(2*pi) + dp.n *((dp.n + tr.Shat) / (dp.n - 2 - tr.Shat))
+    # R2adj <- 1-(1-R2.val)*(dp.n-1)/(edf-1)
+  # }
+  
   if(hatmatrix)
   {
-    tr.Shat <- sum(diag(Shat))
-    tr.StShat<-sum(Shat^2)
-    edf<- dp.n - 2*tr.Shat + tr.StShat
-    AICc <- dp.n*log(sigma.hat21) + dp.n*log(2*pi) + dp.n *((dp.n + tr.Shat) / (dp.n - 2 - tr.Shat))
-    R2adj <- 1-(1-R2.val)*(dp.n-1)/(edf-1)
+    mgwr.diag <- gwr_diag(y, x1, betas, Shat)
+	AIC <- mgwr.diag[[1]]
+	AICc <- mgwr.diag[[2]]
+	edf <- mgwr.diag[[3]]
+	enp <- mgwr.diag[[4]]
+	RSS.gw <- mgwr.diag[[5]]
+	R2.val <- mgwr.diag[[6]]
+	R2adj <- mgwr.diag[[7]]
+	BIC <- mgwr.diag[[10]]
+	GW.diagnostic<-list(RSS.gw=RSS.gw,AICc=AICc,AIC=AIC,BIC=BIC,R2.val=R2.val, R2adj = R2adj, edf=edf, enp=enp)
   }
-  
   #sigma.hat11<-RSS.gw/(dp.n-2*tr.Shat+tr.StShat)
-  GW.diagnostic<-list(RSS.gw=RSS.gw,AICc=AICc,R2.val=R2.val, R2adj = R2adj)
+  
   #########################Retrive the intercept
   if(!is.na(idx1) && n.cent>=1)
   {
@@ -399,7 +426,7 @@ print.multiscalegwr<-function(x, ...)
 	cat("\n   Number of data points:",dp.n)
 	#########################################################################
 	cat("\n   ***********************************************************************\n")
-    cat("   *             GWR with Parameter-Specific Distance Metrics            *\n")
+    cat("   *                       Multiscale (PSDM) GWR                          *\n")
 	cat("   ***********************************************************************\n")
 	cat("\n   *********************Model calibration information*********************\n")
 	cat("   Kernel function:", x$GW.arguments$kernel, "\n")
@@ -432,12 +459,17 @@ print.multiscalegwr<-function(x, ...)
 	cat("   ************************Diagnostic information*************************\n")
 	#rownames(diag.mat) <- c("   Residual sum of squares", "   AICc value:","   R-square value","   Adjusted R-square value")
 	options("scipen"=999)
-	cat("   Residual sum of squares: ", x$GW.diagnostic$RSS.gw, "\n")
-	cat("   R-square value: ", x$GW.diagnostic$R2.val, "\n")
 	if(x$GW.arguments$hatmatrix)
 	{
-	  cat("   Adjusted R-square value: ", x$GW.diagnostic$R2adj, "\n")
+	  cat("   Number of data points:", dp.n, "\n")
+	  cat("   Effective number of parameters (2trace(S) - trace(S'S)):", x$GW.diagnostic$enp, "\n")
+	  cat("   Effective degrees of freedom (n-2trace(S) + trace(S'S)):", x$GW.diagnostic$edf, "\n")
 	  cat("   AICc value: ", x$GW.diagnostic$AICc, "\n")
+	  cat("   AIC value: ", x$GW.diagnostic$AIC, "\n")
+	  cat("   BIC value: ", x$GW.diagnostic$BIC, "\n")
+	  cat("   Residual sum of squares: ", x$GW.diagnostic$RSS.gw, "\n")
+	  cat("   R-square value: ", x$GW.diagnostic$R2.val, "\n")
+	  cat("   Adjusted R-square value: ", x$GW.diagnostic$R2adj, "\n")
 	}
 	cat("\n   ***********************************************************************\n")
 	cat("   Program stops at:", as.character(x$timings$stop), "\n")
@@ -453,7 +485,7 @@ gwr.backfit <- function(x, y, betas,dp.locat,rp.locat,hatmatrix, criterion="CVR"
    dp.n <- nrow(x)
    rp.n <- nrow(rp.locat)
    criterion.val <- 10000000
-   resid.i <- y - gw.fitted(x, betas)
+   resid.i <- y - fitted(x, betas)
    #resid.i <- ehat(y, x, betas)
    RSS0 <- sum(resid.i^2)
    RSS1 <- 0
@@ -485,7 +517,7 @@ gwr.backfit <- function(x, y, betas,dp.locat,rp.locat,hatmatrix, criterion="CVR"
        betas[,i] <- betai
      }
      #RSS1 <- rss(y,x,betas)
-     RSS1 <- sum((y - gw.fitted(x, betas))^2)   
+     RSS1 <- sum((y - fitted(x, betas))^2)   
      if(criterion=="CVR")
      {
          #RSS1 <- sum((y - gwr.fitted(x, betas))^2)
@@ -506,7 +538,7 @@ gwr.backfit <- function(x, y, betas,dp.locat,rp.locat,hatmatrix, criterion="CVR"
    betas
 }
 ###For bandwidth selection
-bw.gwr2<-function(x, y, dp.locat,approach="AIC",kernel="gaussian",adaptive=FALSE,dMat, verbose=F, nlower = 10)
+bw.gwr2<-function(x, y, dp.locat,approach="AIC",kernel="gaussian",adaptive=FALSE,dMat, verbose=F, parallel.method=F,parallel.arg=NULL, nlower = 10)
 {
   dp.n <-  nrow(dp.locat)
   if(adaptive)
@@ -522,11 +554,26 @@ bw.gwr2<-function(x, y, dp.locat,approach="AIC",kernel="gaussian",adaptive=FALSE
   ########################## Now the problem for the golden selection is too computationally heavy
     #Select the bandwidth by golden selection
     bw<-NA
-    if(approach=="cv"||approach=="CV")
-       bw <- gold(gwr.cv,lower,upper,adapt.bw=adaptive,x,y,kernel,adaptive, dp.locat,dMat=dMat,verbose=verbose)
-    else if(approach=="aic"||approach=="AIC"||approach=="AICc")
-       bw <- gold(gwr.aic1,lower,upper,adapt.bw=adaptive,x,y,kernel,adaptive, dp.locat,dMat=dMat,verbose=verbose)
-    bw
+    # ## make cluseter
+  if (parallel.method == "cluster") {
+    if (missing(parallel.arg)) {
+      cl.n <- max(detectCores() - 4, 2)
+      parallel.arg <- makeCluster(cl.n)
+    } else cl.n <- length(parallel.arg)
+    clusterCall(parallel.arg, function() { library(GWmodel) })
+  }
+  # ## call for functions
+  if(approach == "bic" || approach == "BIC")
+      bw <- gold(gwr.bic, lower, upper, adapt.bw = adaptive, x, y, kernel, adaptive, dp.locat, p=2, theta=0, longlat=F, dMat, verbose, parallel.method, parallel.arg)
+  else if(approach == "aic" || approach == "AIC" || approach == "AICc")
+      bw <- gold(gwr.aic, lower, upper, adapt.bw = adaptive, x, y, kernel, adaptive, dp.locat, p=2, theta=0, longlat=F, dMat, verbose, parallel.method, parallel.arg)    
+  else 
+      bw <- gold(gwr.cv, lower, upper, adapt.bw = adaptive, x, y, kernel, adaptive, dp.locat, p=2, theta=0, longlat=F, dMat, verbose, parallel.method, parallel.arg)
+  # ## stop cluster
+  if (parallel.method == "cluster") {
+    if (missing(parallel.arg)) stopCluster(parallel.arg)
+  }
+  bw
 }
 gwr.cv2 <- function(bw, X, Y, kernel,adaptive, dp.locat,dMat)
 {
